"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApolloErrorProcessor = exports.isGraphQLError = exports.isApolloError = void 0;
const types_1 = require("./types");
function isApolloError(error) {
    return error.graphQLErrors !== undefined;
}
exports.isApolloError = isApolloError;
function isGraphQLError(error) {
    return error.extensions !== undefined;
}
exports.isGraphQLError = isGraphQLError;
class ApolloErrorProcessor {
    constructor(error, app, context) {
        this.originalError = error;
        this.app = app;
        this.context = context;
        this.processedErrors = this.processApolloError(error);
    }
    showErrorNotifications() {
        this.processedErrors.forEach(error => {
            console.error(`${error.type}: ${error.message}`, error.error);
        });
    }
    cleanError(error) {
        var _a;
        if (error instanceof Error) {
            return error;
        }
        const cleanError = new Error(error.message);
        if (isGraphQLError(error)) {
            cleanError.name = 'GraphQLError' + (((_a = error.extensions) === null || _a === void 0 ? void 0 : _a.code) != null ? `[${error.extensions.code}]` : '');
            cleanError.stack = this.originalError.stack;
            Object.defineProperty(cleanError, 'nodes', { value: error.nodes });
            Object.defineProperty(cleanError, 'source', { value: error.source });
            Object.defineProperty(cleanError, 'positions', { value: error.positions });
            Object.defineProperty(cleanError, 'path', { value: error.path });
            Object.defineProperty(cleanError, 'extensions', { value: JSON.stringify(error.extensions) });
            Object.defineProperty(cleanError, 'originalError', { value: this.originalError });
        }
        else {
            Object.keys(error).forEach(key => Object.defineProperty(cleanError, key, { value: error[key] }));
        }
        return cleanError;
    }
    isUnauthorizedError(error) {
        var _a, _b, _c;
        return (error.message === 'Unauthorized' ||
            ((_a = error.extensions) === null || _a === void 0 ? void 0 : _a.code) === 'FORBIDDEN' ||
            ((_c = (_b = error.extensions) === null || _b === void 0 ? void 0 : _b.exception) === null || _c === void 0 ? void 0 : _c.status) === 401);
    }
    onUnauthorizedError(error) {
    }
    onServerError(error) {
    }
    onInputValidationError(error) {
    }
    getFriendlyMessage(errorCode, errorMessage) {
        var _a;
        return (_a = this.constructor.FriendlyMessages[errorCode]) !== null && _a !== void 0 ? _a : errorMessage;
    }
    processErrorMessage(message) {
        if (typeof message === 'object') {
            if (message.error != null) {
                return message.error;
            }
            return 'Unknown error: ' + JSON.stringify(message);
        }
        return message;
    }
    normalizeError(error) {
        if (isGraphQLError(error)) {
            return {
                ...error,
                message: this.processErrorMessage(error.message),
            };
        }
        return error;
    }
    processApolloError(error) {
        if (error.graphQLErrors != null && error.graphQLErrors.length > 0) {
            return error.graphQLErrors.flatMap(graphQLError => this.processGraphQLError(graphQLError));
        }
        if (error.networkError != null &&
            error.networkError.result != null &&
            error.networkError.result.errors != null &&
            error.networkError.result.errors.length > 0) {
            return error.networkError.result.errors.flatMap(graphQLError => this.processGraphQLError(graphQLError));
        }
        if (error.networkError != null) {
            return this.processNetworkError(error);
        }
        const processedError = {
            type: types_1.ApolloErrorType.SERVER_ERROR,
            error,
            message: error.message,
        };
        this.onServerError(processedError);
        return [processedError];
    }
    processGraphQLError(error) {
        var _a, _b;
        if (this.isUnauthorizedError(error)) {
            const processedError = {
                type: types_1.ApolloErrorType.UNAUTHORIZED_ERROR,
                error: this.normalizeError(error),
                message: this.processErrorMessage(error.message),
                path: error.path,
            };
            this.onUnauthorizedError(processedError);
            return [processedError];
        }
        if (((_a = error.extensions) === null || _a === void 0 ? void 0 : _a.validationErrors) != null) {
            const processedError = {
                type: types_1.ApolloErrorType.INPUT_VALIDATION_ERROR,
                error: this.normalizeError(error),
                message: this.processErrorMessage(error.message),
                path: error.path,
                invalidArgs: error.extensions.invalidArgs,
                violations: error.extensions.validationErrors,
            };
            this.onInputValidationError(processedError);
            return [processedError];
        }
        const processedError = {
            type: ((_b = error.extensions) === null || _b === void 0 ? void 0 : _b.code) != null ? error.extensions.code : types_1.ApolloErrorType.SERVER_ERROR,
            error: this.normalizeError(error),
            message: this.getFriendlyMessage('INTERNAL_SERVER_ERROR', this.processErrorMessage(error.message)),
            path: error.path,
        };
        this.onServerError(processedError);
        return [processedError];
    }
    processNetworkError(error) {
        const errors = [];
        let message;
        if (error.networkError != null && error.networkError.message != null) {
            message = this.processErrorMessage(error.networkError.message);
        }
        else {
            message = this.processErrorMessage(error.message);
        }
        switch (message) {
            case 'Failed to fetch':
                message = this.getFriendlyMessage('FAILED_TO_FETCH', message);
                break;
        }
        errors.push({
            type: types_1.ApolloErrorType.NETWORK_ERROR,
            error,
            statusCode: error.networkError != null ? error.networkError.statusCode : undefined,
            message,
        });
        return errors;
    }
}
exports.ApolloErrorProcessor = ApolloErrorProcessor;
ApolloErrorProcessor.FriendlyMessages = {
    FAILED_TO_FETCH: 'Unable to communicate with server. The service may be down or you may be offline. Try again in a moment.',
    INTERNAL_SERVER_ERROR: `A server error has occurred.`,
};
//# sourceMappingURL=ApolloErrorProcessor.js.map